/* This file was generated by syncqt. */
#ifndef QT_QTTEXTTOSPEECH_VERSION_H
#define QT_QTTEXTTOSPEECH_VERSION_H

#define QTTEXTTOSPEECH_VERSION_STR "5.9.5"

#define QTTEXTTOSPEECH_VERSION 0x050905

#endif // QT_QTTEXTTOSPEECH_VERSION_H
